#include "uwbgl_D3DGraphicsSystem2.h"

UWBD3D_GraphicsSystem UWBD3D_GraphicsSystem::m_TheAPI;

// return the one and only GraphicsSystemD3D instance
UWBD3D_GraphicsSystem& UWBD3D_GraphicsSystem::GetSystem()
{
  return m_TheAPI;
}

UWBD3D_GraphicsSystem::UWBD3D_GraphicsSystem()
: m_pD3D(NULL)
, m_pD3DDevice(NULL)
, m_SwapChainCounter(0)
{
}

UWBD3D_GraphicsSystem::~UWBD3D_GraphicsSystem()
{
  ShutDown();
}

bool UWBD3D_GraphicsSystem::CreateGraphicsContext(HWND hMainWindow)
{
  // return true if a graphics context has already been created and assigned to m_pD3DDevice
  if( m_pD3DDevice && m_pD3D )
    return true;  //already initialized

  // create link to the Direct3D graphics system
  m_pD3D = Direct3DCreate9( D3D_SDK_VERSION);
  //check that abstract link created successfully
  if(NULL == m_pD3D)
    return false;

  // Set up the structure used to create the D3dDevice.
  // Init the entire D3DPRESENT structure to 0
  D3DPRESENT_PARAMETERS present_params;
  ZeroMemory( &present_params, sizeof(present_params) );

  // Set Windowed to TRUE, since we want to do D3d in a window
  present_params.Windowed = TRUE;

  // set SwapEffect to "discard", which is the most efficient method 
  // of presenting the back buffer to the display.
  present_params.SwapEffect = D3DSWAPEFFECT_DISCARD;

  // request a back buffer format that matches the current desktop display format
  D3DDISPLAYMODE display_mode;
  m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &display_mode );
  present_params.BackBufferFormat = display_mode.Format;

  present_params.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

  // Enable Z-buffer stenciling to support 3D ..
  // in the future, we will have to do more here, but for now, for 2D this is sufficient.
  // go check out the D3D on-line manual to what other options are and what they mean.

  // Create the Direct3D device. Here we are using the default adapter (most
  // systems only have one, unless they have multiple graphics hardware cards
  // installed) and requesting the HAL (which is saying we want the hardware
  // device rather than a software one). Software vertex processing is 
  // specified since we know it will work on all cards. On cards that support 
  // hardware vertex processing, though, we would see a big performance gain 
  // by specifying hardware vertex processing.
  if( FAILED(m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,  hMainWindow,
                                   D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                   &present_params, &m_pD3DDevice ) ) )
  {
    MessageBox(NULL, L"D3D Creation Failed during CreateDevice", L"UWBD3D_GraphicsSystem::CreateGraphicsContext", MB_OK);
    return false;
  }

  m_pD3DDevice->SetFVF(D3DFVF_XYZ | D3DFVF_DIFFUSE);
  m_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
  m_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

  // Set the matrices to identity
  D3DXMATRIX identity;
  D3DXMatrixIdentity(&identity);
  m_pD3DDevice->SetTransform( D3DTS_WORLD, &identity);
  m_pD3DDevice->SetTransform( D3DTS_VIEW, &identity);
  m_pD3DDevice->SetTransform( D3DTS_PROJECTION, &identity);

  return true;
}

// Release all of the connections to Direct3D
void UWBD3D_GraphicsSystem::ShutDown()
{
  // Direct3D is implemented using COM techniques. We never actually
  // "delete" the pointers we get from Direct3D. Instead we tell Direct3D
  // that we are done using an object by calling Release() on it.

  //destroy the graphics device   
  if(m_pD3DDevice)
  {
    m_pD3DDevice->Release();
    m_pD3DDevice = NULL;
  }

  //release connection to Direct3D
  if(m_pD3D)
  {
    m_pD3D->Release();
    m_pD3D = NULL;
  }
}

//  Check to see if the graphics system is initialized
bool UWBD3D_GraphicsSystem::IsInitialized() const
{
  return (NULL!=m_pD3D && NULL!=m_pD3DDevice);
}

LPDIRECT3DDEVICE9 UWBD3D_GraphicsSystem::GetD3DDevice()
{
  return m_pD3DDevice;
}

//  Method used to create the swap chain for the device.
//  @return LPDIRECT3DSWAPCHAIN9 The swap chain pointer obj
LPDIRECT3DSWAPCHAIN9 UWBD3D_GraphicsSystem::CreateSwapChain(HWND hWindow)
{
  if(!IsWindow(hWindow))
    return NULL;

  RECT client_rect;
  GetClientRect(hWindow, &client_rect);
  int buffer_width = client_rect.right - client_rect.left;
  int buffer_height = client_rect.bottom - client_rect.top;
  if(0 == buffer_width || 0 == buffer_height)
    return NULL;

  LPDIRECT3DSWAPCHAIN9 pSwapChain = NULL;

  // because the first swap chain is created when the device is created
  // we just have to get it for the first one
  if (m_SwapChainCounter == 0)
  {
    m_pD3DDevice->GetSwapChain( 0, &pSwapChain );
  }
  // for all swap chains after the first one, the device must create them
  else 
  {
    D3DDISPLAYMODE d3ddm;   //the display mode for the adapter
    m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm );
    D3DPRESENT_PARAMETERS   pp;  // This is the D3D device description
    ZeroMemory( &pp, sizeof(pp) );
    pp.Windowed = TRUE;
    pp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    pp.BackBufferFormat = d3ddm.Format;
    pp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
    pp.hDeviceWindow = hWindow;
    pp.BackBufferWidth = buffer_width;
    pp.BackBufferHeight = buffer_height;
    m_pD3DDevice->CreateAdditionalSwapChain( &pp, &pSwapChain );
  }

  m_SwapChainCounter++;
  return pSwapChain;
}