// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Common.h"

/// Window Handlers allow for hooking a graphics API to a window (i.e. HWND)

/// The UWB_IWindowHandler struct acts as a pure abstract base class for API specific windows. This
/// is done because this is the one item that must be shared across our different libraries.
/// By having absolutely no implementation, no project dependencies are created.
/// A graphics window can contain an UWB_IWindowHandler. Mouse and graphics drawing calls are sent to
/// this UWB_IWindowHandler from the graphics window on appropriate system events.
struct UWB_IWindowHandler
{
  virtual ~UWB_IWindowHandler(){}

  /// Check that a valid connection to Direct3D, OpenGL, or some other API specific graphics
  /// technology exists for this window
  virtual bool IsGraphicsDeviceValid() = 0;

  /// When a handler is attached to a graphics window or right after an attached window
  /// is created, the window will call this function. This allows the handler to attempt
  /// to set itself up and attain resources from whatever graphics technology
  /// it is working with.
  /// \param hAttachedWindow [in]: window handle for the graphics window that
  ///                              this is being attached to
  /// \return true on success
  virtual bool InitializeHandler(HWND hAttachedWindow)=0;

  /// Called right before a window is destroyed, or when it thinks that it does
  /// not need to use this handler.
  virtual void ShutDownHandler()=0;

  /// Perform all drawing operations for this window
  virtual void BeginDraw()=0;
  virtual void EndDrawAndShow()=0;
  virtual void DrawGraphics()=0;



  ///// Mouse Events /////
  /// The attached graphics window passes mouse events to its viewport. This makes dealing
  /// with mouse input much easier as you only have to subclass a single viewport class to
  /// perform custom drawing and mouse code

  /// Called when the mouse is clicked down or up on a graphics window
  /// \param down [in]: true if the mouse is clicked down
  /// \param nFlags [in]: indicates which mouse button and keys are down, can be a combination
  ///                     of any of the following: MK_CONTROL, MK_LBUTTON, MK_MBUTTON, MK_RBUTTON, MK_SHIFT
  ///                     These are defined in winuser.h
  /// \param hwX, hwY [in]: client window coordinates of the cursor
  virtual void OnMouseButton(bool down, unsigned int nFlags, int hwX, int hwY)=0;

  /// Called when the mouse is moved over a graphics window
  /// \param nFlags [in]: indicates which mouse button and keys are down, can be a combination
  ///                     of any of the following: MK_CONTROL, MK_LBUTTON, MK_MBUTTON, MK_RBUTTON, MK_SHIFT
  ///                     These are defined in winuser.h
  /// \param hwX, hwY [in]: client window coordinates of the cursor
  virtual void OnMouseMove(unsigned int nFlags, int hwX, int hwY)=0;
};
