#include "uwbgl_MFCUIWindow1.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

// Standard MFC macro or defining handler functions for different window's messages
BEGIN_MESSAGE_MAP(UWBMFC_UIWindow, CWnd)
  ON_WM_ERASEBKGND()
  ON_WM_PAINT()
  ON_WM_LBUTTONDOWN()
  ON_WM_MBUTTONDOWN()
  ON_WM_RBUTTONDOWN()
  ON_WM_MOUSEMOVE()
  ON_WM_LBUTTONUP()
  ON_WM_MBUTTONUP()
  ON_WM_RBUTTONUP()
END_MESSAGE_MAP()

UWBMFC_UIWindow::UWBMFC_UIWindow(bool timerBased)
: m_bTimerBased(timerBased)
, m_pHandler(NULL)
{
}

UWBMFC_UIWindow::~UWBMFC_UIWindow()
{
}

bool UWBMFC_UIWindow::ReplaceDialogControl(HWND hDialogHandle, int id)
{
  //check for valid input parameters
  CWnd* pDialog = CWnd::FromHandle(hDialogHandle);
  ASSERT(pDialog!=NULL);
  CWnd* pCurrentCtrl = pDialog->GetDlgItem(id);
  if(NULL==pCurrentCtrl) return false;

  //Copy the contol's styles. This gives us some way to define things like border style
  //in the resource editor
  DWORD style = pCurrentCtrl->GetStyle();
  DWORD styleEx = pCurrentCtrl->GetExStyle();
  // use the control to figure out how large is the initial on-screen area for the graphics window
  CRect rect;
  pCurrentCtrl->GetWindowRect(&rect);

  pCurrentCtrl->DestroyWindow();  // Since the static was just a place holder, destroy it
                                  // convert the static's screen rectangle to the dialog's 
                                  // client coordinates
  
  
  // use the on-screen area information to create our UWBMFC_UIWindow
  pDialog->ScreenToClient(&rect);
  BOOL success = CreateEx(styleEx,
                          NULL,     //CWnd default
                          NULL,     //has no name
	                  style|WS_CHILD|WS_CLIPSIBLINGS|WS_CLIPCHILDREN|WS_VISIBLE,
	                  rect,     // Dimension and location of the place-holder
	                  pDialog,  //this is the parent
	                  id);      //Use the ID of the just destroyed static

  BringWindowToTop();
  if(!success)
    ::MessageBox(hDialogHandle, L"UWBMFC_UIWindow creation failed!", L"UWBMFC_UIWindow::ReplaceDialogControl", MB_OK|MB_ICONSTOP);

  return (success==TRUE);
}

/// This just overrides the default window behavior of erasing the background of window. If we
/// don't override this behavior, the window will procude a "blinking" effect.
BOOL UWBMFC_UIWindow::OnEraseBkgnd(CDC* pDC)
{
  if(m_pHandler)
    return TRUE;

  return CWnd::OnEraseBkgnd(pDC);
}

/// Handler function for a WM_PAINT message. This is where standard windows draw themselves.
/// Most CSS450/451 windows will draw during a timer event, so we typically won't draw anything
/// at this time.
void UWBMFC_UIWindow::OnPaint()
{
  CPaintDC dc(this); // device context for painting
  if( NULL == m_pHandler )
  {
    //If we haven't set up a handler, draw a message that no viewport has been attached.
    CRect client_rect;
    GetClientRect(client_rect);
    CBrush white_brush(RGB(255,255,255));
    dc.FillRect(client_rect, &white_brush);

    CPen pen(PS_SOLID,1,RGB(255,255,255));
    CPen* pOldPen = dc.SelectObject(&pen);
    int bk_mode = dc.SetBkMode(TRANSPARENT);
    dc.DrawText(_T("No Graphics Handler Attached"),&client_rect,DT_CENTER|DT_VCENTER|DT_SINGLELINE);
    dc.SetBkMode(bk_mode);
    dc.SelectObject(pOldPen);
  }
  else
  {
    //You could optionally set up a renderer to draw on standard window's paint messages.
    //Most renderers in CSS450/451 are going to be timer based for animation purposes.
    if(!m_bTimerBased)
      m_pHandler->DrawGraphics();
  }
}

/// Left mouse button down handler. Calls OnMouseButton virtual function of the contained viewport.
void UWBMFC_UIWindow::OnLButtonDown(UINT nFlags, CPoint point)
{
  if(m_pHandler)
  {
    //Tell windows that this window should get all mouse messages until capture is released
    SetCapture();
    m_pHandler->OnMouseButton(true,nFlags,point.x,point.y);
  }
}

/// Middle mouse button down handler. Calls OnMouseButton virtual function of the contained viewport.
void UWBMFC_UIWindow::OnMButtonDown(UINT nFlags, CPoint point)
{
  if(m_pHandler)
  {
    //Tell windows that this window should get all mouse messages until capture is released
    SetCapture();
    m_pHandler->OnMouseButton(true,nFlags,point.x,point.y);
  }
}

/// Right mouse button down handler. Calls OnMouseButton virtual function of the contained viewport.
void UWBMFC_UIWindow::OnRButtonDown(UINT nFlags, CPoint point)
{
  if(m_pHandler)
  {
    //Tell windows that this window should get all mouse messages until capture is released
    SetCapture();
    m_pHandler->OnMouseButton(true,nFlags,point.x,point.y);
  }
}

/// Mouse movde handler. Calls OnMouseMove virtual function of the contained viewport.
void UWBMFC_UIWindow::OnMouseMove(UINT nFlags, CPoint point)
{
  if(m_pHandler)
  {
    m_pHandler->OnMouseMove(nFlags,point.x,point.y);
  }
}

/// Left mouse button up handler. Calls OnMouseButton virtual function of the contained viewport.
void UWBMFC_UIWindow::OnLButtonUp(UINT nFlags, CPoint point)
{
  if(m_pHandler)
  {
    nFlags |= MK_LBUTTON;
    ReleaseCapture();
    m_pHandler->OnMouseButton(false,nFlags,point.x,point.y);
  }
}
/// Middle mouse button up handler. Calls OnMouseButton virtual function of the contained viewport.
void UWBMFC_UIWindow::OnMButtonUp(UINT nFlags, CPoint point)
{
  if(m_pHandler)
  {
    nFlags |= MK_MBUTTON;
    ReleaseCapture();
    m_pHandler->OnMouseButton(false,nFlags,point.x,point.y);
  }
}
/// Right mouse button up handler. Calls OnMouseButton virtual function of the contained viewport.
void UWBMFC_UIWindow::OnRButtonUp(UINT nFlags, CPoint point)
{
  if(m_pHandler)
  {
    nFlags |= MK_RBUTTON;
    ReleaseCapture();
    m_pHandler->OnMouseButton(false,nFlags,point.x,point.y);
  }
}

bool UWBMFC_UIWindow::AttachHandler(UWB_IWindowHandler* pHandler)
{
  if(pHandler==m_pHandler)
    return true;

  bool attachSuccess = true;

  if(IsWindow(m_hWnd))
  {
    if(m_pHandler)
    {
      m_pHandler->ShutDownHandler();
      m_pHandler = NULL;
    }
    attachSuccess = pHandler->InitializeHandler(m_hWnd);
  }

  if(attachSuccess)
    m_pHandler = pHandler;
  else
    MessageBox(L"Error attaching viewport to window!", L"UWBMFC_UIWindow::AttachViewport", MB_OK|MB_ICONSTOP);

  return attachSuccess;
}

BOOL UWBMFC_UIWindow::PreCreateWindow(CREATESTRUCT& cs)
{
  cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS | CS_OWNDC
                                     , LoadCursor(NULL, IDC_ARROW)
                                     , NULL
                                     , NULL);
  cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  return __super::PreCreateWindow(cs);
}
