// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_MFCDefines.h"
#include "../UWBGL_Direct3D/uwbgl_IWindowHandler.h"

/// An MFC derived CWnd (window) that supports containment of a Viewport. Default implementation
/// allows for a single viewport that covers the entire window area. Mouse, draw, and resize
/// events are passed down to contained viewport.
/// The class is designed to be placed on a dialog by replacing a "placeholder"
/// type control on the dialog.
/// \see MSDN documentation on the MFC CWnd class
///      http://msdn2.microsoft.com/en-us/library/1xb05f0h(VS.80).aspx
class UWBMFC_UIWindow : public CWnd
{
public:
  /// Constructor. Note that this does not actually create a window that you see on
  /// the screen. Window creation is done by either calling the Create(...) or
  /// ReplaceDialogControl(...) functions.
  /// \param timerBase If true, the window does not repaint on normal paint messages. All repaint
  ///                  functionality is driven through an external timer function. This is the
  ///                  typical approach for CSS450/410 projects.
  UWBMFC_UIWindow(bool timerBased = true);

  /// The destructor does NOT destroy it's viewport. It is up to the project to destroy the actual
  /// viewport object.
  virtual ~UWBMFC_UIWindow();

  /// This class is designed to be placed on a window's dialog by creating a "placerholder"
  /// control in the dialog editor and then placing this window in the location of the placeholder.
  /// This is done by calling ReplaceDialogControl in the dialog's OnInitDialog routine.
  /// Example:
  ///   BOOL CTutorialDlg::OnInitDialog()
  ///   {
  ///     CDialog::OnInitDialog();
  ///     m_GrfxWnd.ReplaceDialogControl(m_hWnd, IDC_GRFX_PLACEHOLDER);
  ///     return TRUE;  // return TRUE  unless you set the focus to a control
  ///   }
  ///
  /// \param  hDialogHandle handle to the parent dialog
  /// \param  id ID for the placeholder control that was set on the dialog in the dialog editor
  /// \return true on successful replacement
  bool ReplaceDialogControl(HWND hDialogHandle, int id);

  /// Set a handler class that this window will pass drawing and mouse events to
  /// Note: The handler should exist as long as this window exists or pass NULL to this window when
  /// the handler is being removed/destroyed
  /// \param pViewport The handler viewport. The lifetime of the handler is not controled by this class
  /// \return true on success
  bool AttachHandler(UWB_IWindowHandler* pViewport);

protected:
  /// A message map is an MFC macro for mapping window's events (paint, size, mouse,...) that
  /// occur on a window to functions in this class.
  /// The functions in this class that begin with afx_msg are "handlers" for window's messages
  DECLARE_MESSAGE_MAP()

  // windows event handler functions. most functions will end up calling a
  // virtual function on the Viewport

  /// Called when this window receives a WM_ERASEBKGND message (event)
  /// Typical windows fill the background with some brush, but when writing
  /// graphics windows that draw everything at once, there is no need to clear
  /// the background (which would cause a flicker effect.) This function always
  /// just returns without clearing the background
  afx_msg BOOL OnEraseBkgnd(CDC* pDC);

  /// Called when this window receives a WM_PAINT message (event)
  /// Windows has notified this window that it needs to be redrawn.
  afx_msg void OnPaint();

  // mouse messages that are forwarded to the handler class
  afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
  afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
  afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
  afx_msg void OnMouseMove(UINT nFlags, CPoint point);
  afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
  afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
  afx_msg void OnRButtonUp(UINT nFlags, CPoint point);

  /// Called just prior to the actual window creation. This class represents a window
  /// and hooks to a HWND handle that Win32 creates. Prior to the window creation, this
  /// class gets an opportunity to step in and modify the style flags used by the Win32
  /// CreateWindowEx(...) function.
  /// OpenGL windows require a window's style of CS_OWNDC. This setting is also
  /// fine for DirectX windows.
  /// \param cs modify this structure to modify the window creation style flags
  /// \return TRUE if the window should be created
  virtual BOOL PreCreateWindow(CREATESTRUCT& cs);

protected:
  /// The event handler class. Windows messages are forwarded to this class
  /// to take the appropriate actions
  UWB_IWindowHandler* m_pHandler;

  /// If true, this window's drawing is based on a timer event. If false, the window
  /// should redraw when it gets a WM_PAINT message
  bool m_bTimerBased;
};
